# MIT License

# Copyright (c) 2025 Institute for Automotive Engineering (ika), RWTH Aachen University

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
GUI Widgets Package.

Provides custom Qt widgets for the ros2_unbag graphical user interface:
- TopicListWidget: Displays and manages topic selection
- TopicSettingsWidget: Configures per-topic export options
- GlobalSettingsWidget: Manages global export settings
- ProcessorChainWidget: Builds ordered processor pipelines
"""

from .processor_chain import ProcessorChainWidget
from .topic_list import TopicListWidget
from .topic_settings import TopicSettingsWidget
from .global_settings import GlobalSettingsWidget

__all__ = [
    "ProcessorChainWidget", 
    "TopicListWidget",
    "TopicSettingsWidget",
    "GlobalSettingsWidget"
]
